<?php
namespace wcf\data\shoutbox;
use wcf\data\DatabaseObject;
use wcf\system\WCF;
use wcf\system\bbcode\SimpleMessageParser;
use wcf\util\StringUtil;

/**
 * Shoutbox entry
 * 
 * @author	Sonnenspeer
 * @copyright	2013 Sonnenspeer
 * @license	GNU Lesser General Public License <http://opensource.org/licenses/lgpl-license.php>
 * @package	de.sonnenspeer.shoutbox
 */
class ShoutboxEntry extends DatabaseObject {
    /**
     * @see    wcf\data\DatabaseObject::$databaseTableName
     */
    protected static $databaseTableName = 'shoutbox_entry';
    
    /**
     * @see    wcf\data\DatabaseObject::$databaseTableIndexName
     */
    protected static $databaseTableIndexName = 'entryID';
	
	/**
	 * Creates a new ShoutboxEntry object.
	 * 
	 * @param 	integer 	$entryID
	 * @param 	array 		$row
	 */
	public function __construct($entryID, $row = null, DatabaseObject $object = null) {		
		if ($entryID !== null) {			
			$sql = "SELECT	* 
				FROM	wcf".WCF_N."_shoutbox_entry shoutbox_entry
				WHERE	entryID = ?";
			
			$statement = WCF::getDB()->prepareStatement($sql);
			$statement->execute(array($entryID));
			$row = $statement->fetchArray();			
			
            // enforce data type 'array'
            if ($row === false) $row = array();
        }
        else if ($object !== null) {    	
            $row = $object->data;
        }
		
		$row['isDeletable'] = WCF::getSession()->getPermission('user.shoutbox.canDeleteOwnEntry');		

        $this->handleData($row);
		
		if(!isset($this->userOnlineMarking)) $this->userOnlineMarking = '%s';
	}
	
	/**
	 * Returns true, if active user may delete entry
	 * 
	 * @return	boolean
	 */
	public function isDeletable() {
		if (($this->userID && $this->userID == WCF::getUser()->userID && WCF::getSession()->getPermission('user.shoutbox.canDeleteOwnEntry')) || WCF::getSession()->getPermission('mod.shoutbox.canDeleteEntry')) {
			return true;
		}
		return false;
	}

	/**
	 * Returns formatted username style.
	 * 
	 * @return	string
	 */
	public function getFormattedUsername() {
		if ($this->userOnlineMarking != '%s') {
			return sprintf($this->userOnlineMarking, StringUtil::encodeHTML($this->username));
		}
		return StringUtil::encodeHTML($this->username);
	}
	
	/**
	 * Returns a formatted message.
	 * 
	 * @return 	string
	 */	
	public function getFormattedMessage() {	
		return SimpleMessageParser::getInstance()->parse($this->message);	
	}
}
?>