<?php
namespace wcf\data\shoutbox;
use wcf\data\AbstractDatabaseObjectAction;
use wcf\system\WCF;
use wcf\data\shoutbox\ShoutboxEntryEditor;
use wcf\util\StringUtil;
use wcf\system\exception\NamedUserException;
use wcf\system\exception\IllegalLinkException;
use wcf\util\HeaderUtil;
use wcf\util\UserUtil;
use wcf\util\JSON;
use wcf\system\bbcode\SimpleMessageParser;
use wcf\system\database\util\PreparedStatementConditionBuilder;

/**
 * Shoutbox actions
 * 
 * @author	Sonnenspeer
 * @copyright	2013 Sonnenspeer
 * @license	GNU Lesser General Public License <http://opensource.org/licenses/lgpl-license.php>
 * @package	de.sonnenspeer.shoutbox
 */
class ShoutboxEntryAction extends AbstractDatabaseObjectAction {
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction::$className
	 */
	protected $className = 'wcf\data\shoutbox\ShoutboxEntryEditor';
	
	/**
     * @see    wcf\data\AbstractDatabaseObjectAction::$allowGuestAccess
     */
    protected $allowGuestAccess = array('add', 'loadEntries', 'insertEntries');
	
	/**
	 * Array of entries
	 * @var array
	 */
	public $entries = array();	
	
	public function validateAdd() { }
	
	/**
	 * Add entry
	 */
	public function add() {
		WCF::getSession()->checkPermissions(array('user.shoutbox.canAddEntry'));
		
		// flood control	
		if (WCF::getSession()->getPermission('user.shoutbox.floodControlTime')) {
			$sql = "SELECT		time
				FROM		wcf".WCF_N."_shoutbox_entry
				WHERE		".(WCF::getUser()->userID ? "userID = ".WCF::getUser()->userID : "ipAddress = '".escapeString(WCF::getSession()->ipAddress)."'")."
						AND time > ".(TIME_NOW - WCF::getSession()->getPermission('user.shoutbox.floodControlTime'))."
				ORDER BY	time DESC";
			$statement = WCF::getDB()->prepareStatement($sql);
			$statement->execute();
			$row = $statement->fetchArray();
			if (isset($row['time'])) {
				throw new NamedUserException(WCF::getLanguage()->getDynamicVariable('wcf.shoutbox.entry.error.floodControl', array(
					'waitingTime' => $row['time'] - (TIME_NOW - WCF::getSession()->getPermission('user.shoutbox.floodControlTime')),
					'floodControlTime' => WCF::getSession()->getPermission('user.shoutbox.floodControlTime')
				)));
			}
		}
		
		// get username
		if (isset($this->parameters['data']['username'])) {
			$username = StringUtil::trim($this->parameters['data']['username']);
		}
		if (WCF::getUser()->userID == 0) {
			if (empty($username)) {
				throw new NamedUserException(WCF::getLanguage()->get('wcf.shoutbox.entry.error.username.empty'));
			}
			elseif (!UserUtil::isValidUsername($username)) {
				throw new NamedUserException(WCF::getLanguage()->get('wcf.user.username.error.notValid'));
			}
			elseif (!UserUtil::isAvailableUsername($username)) {
				throw new NamedUserException(WCF::getLanguage()->get('wcf.user.username.error.notUnique'));
			}
		}
		else {
			$username = WCF::getUser()->username;
		}
		
		// get message
		if (isset($this->parameters['data']['message'])) {
			$message = StringUtil::trim($this->parameters['data']['message']);
		}
		if (empty($message)) {
			throw new NamedUserException(WCF::getLanguage()->get('wcf.shoutbox.entry.error.message.empty'));
		}
		if (StringUtil::length($message) > WCF::getSession()->getPermission('user.shoutbox.maxEntryLength')) {
			throw new NamedUserException(WCF::getLanguage()->getDynamicVariable('wcf.message.error.tooLong', array('maxTextLength' => WCF::getSession()->getPermission('user.shoutbox.maxEntryLength'))));
		}
		$message = StringUtil::replace("\n", '', StringUtil::unifyNewlines($message));	
		
		// insert shoutbox entry
		$sql = "INSERT INTO	wcf".WCF_N."_shoutbox_entry
					(userID, username, time, message, ipAddress)
			VALUES		(?, ?, ?, ?, ?)";
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute(array(
			WCF::getUser()->userID,
			escapeString($username),
			TIME_NOW,
			escapeString($message),
			escapeString(WCF::getSession()->ipAddress)
		));
		
	}

	public function validateDelete() { }

	/**
	 * Delete selected entry
	 */
	public function delete() {
		// get shoutbox entry
		if (isset($this->parameters['data']['entryID'])) $entryID = intval($this->parameters['data']['entryID']);
		$entry =  new ShoutboxEntry($entryID);
		
		// legal entry?
		if (!$entry->entryID) {
			throw new IllegalLinkException();
		}
		
		// delete
		$this->deleteAll($entry->entryID);	
	}
	
	public function validateDeleteAll() { }

	/**
	 * Delete all entries
	 */
	public function deleteAll($entryIDs) {
		if (empty($entryIDs)) return;
		
		// delete entries
		$sql = "DELETE FROM	wcf".WCF_N."_shoutbox_entry
			WHERE		entryID IN (".$entryIDs.")";
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute();
	}
	
	public function validateLoadEntries() {}
	
	/**
	 * Load all new entries 
	 * @return json
	 */
	public function loadEntries() {
		if (isset($this->parameters['data']['entryID'])) $entryID = intval($this->parameters['data']['entryID']);
		
		// init shoutbox entry factory
		$factory = new ShoutboxEntryFactory(false);
		$entryList = $factory->entryList;
		$entryList->getConditionBuilder()->add('entryID >= ?', array($entryID));
		$factory->init();
		
		// get shoutbox entries
		$entries = $factory->getEntries();
		$result = $resultArray = array();
		
		$result['entries'] = array();

		foreach($entries AS $entry) {
			//print_r($entry);
			$result['entries'][$entry->entryID] = $entry->getData();
			$result['entries'][$entry->entryID]['message'] = SimpleMessageParser::getInstance()->parse($entry->message);
		}
		
		$resultArray['entries'] = array_values(array_filter($result['entries'])); 

		// send JSON-encoded response
		header('Content-type: application/json');
		echo JSON::encode($resultArray);
		exit;
	}
}
