<?php
namespace wcf\data\shoutbox;
use wcf\system\event\EventHandler;
use wcf\system\WCF;
use wcf\system\cache\builder\SmileyCacheBuilder;
use wcf\data\smiley\SmileyList;

/**
 * Shoutbox factory
 * 
 * @author	Sonnenspeer
 * @copyright	2013 Sonnenspeer
 * @license	GNU Lesser General Public License <http://opensource.org/licenses/lgpl-license.php>
 * @package	de.sonnenspeer.shoutbox
 */
class ShoutboxEntryFactory {
	/**
	 * number of max entries
	 * 
	 * @var	integer
	 */
	public $maxEntries = SHOUTBOX_MAX_ENTRIES;
	
	/**
	 * True, if the smileys should be loaded.
	 * 
	 * @var	boolean
	 */
	public $enableSmileyList = true;
	
	/**
	 * list of shoutbox entries
	 * 
	 * @var	ShoutboxEntryList
	 */
	public $entryList = array();
	
	/**
	 * list of smileys
	 * 
	 * @var array<Smiley>
	 */
	public $smileys = array();
	
	/**
	 * Creates a new ShoutboxEntryFactory.
	 * 
	 * @param	boolean		$enableSmileyList
	 */
	public function __construct($enableSmileyList = true) {
		$this->enableSmileyList = $enableSmileyList;
		
		// init shoutbox entry list
		$this->entryList = new ShoutboxEntryList();
	}
	
	/**
	 * Initializes the shoutbox entries.
	 */
	public function init() {
		EventHandler::getInstance()->fireAction($this, 'shouldInit');

		$smileyIDs = SmileyCachebuilder::getInstance()->getData();
		
		// read shoutbox entries
		$this->entryList->sqlLimit = $this->maxEntries;
		$this->entryList->readObjects();
		
		// get smileys
		if ($this->enableSmileyList) {
			$smileyList = new SmileyList();
			$smileyList->readObjects();	
			$this->smileys = $smileyList;
		}

		EventHandler::getInstance()->fireAction($this, 'didInit');
	}
	
	/**
	 * Returns shoutbox entries.
	 * 
	 * @return	array<ViewableShoutboxEntry>
	 */
	public function getEntries() {
		return array_reverse($this->entryList->getObjects());
	}
	
	/**
	 * Returns smileys.
	 * 
	 * @return	array<Smiley>
	 */
	public function getSmileys() {
		return $this->smileys;
	}
}
?>